#!/usr/bin/env ruby

# if contents.bom in current directory, upload by sftp to math.columbia.edu
# http://www.rubydoc.info/gems/net-sftp
# gem install net-sftp

require 'net/sftp'

server = 'math.columbia.edu'
user = 'bayer'
keys = ["/Users/dave/.ssh/id_rsa"]
local_prefix = '/Volumes/All/Local/Web/math.columbia.edu/'
remote_prefix = '/bayer/html/'
bomfile = 'contents.bom'

####################################################

# get file hash, dir list from bomfile
def list_bom(bomfile)
  files = {}
  %x{ lsbom -f -p fcs #{bomfile} }.each_line do |line|
    file, check, size = line.chomp.split("\t")
    file.sub!('./', '')
    next if file =~ %r{(^|/)[.]}
    files[file] = [check, size]
  end
  dirs = []
  %x{ lsbom -d -s #{bomfile} }.split("\n").each do |dir|
    dir.sub!('./', '')
    next if dir =~ %r{(^|/)[.]}
    dirs << dir   
  end
  [files, dirs]
end

# get dir list from bomfile
def dir_list(bomfile)
  %x{ lsbom -d -s #{bomfile} }.split("\n")
end

# update server
def update(tick, who, &block)
  puts "#{tick} #{who}"
  begin
    yield
  rescue
    puts "    sftp error"
  end
end

# create nested directory on server

def mkdir(sftp, remote_path)
  partials = remote_path.split('/')[1..-1]
    .inject(['']) {|path,item| path.push "#{path.last}/#{item}" }
  partials[1..-1].each do |path|
    begin
      sftp.stat! path
    rescue
      puts "+ #{path}"
      begin
        sftp.mkdir! path
      rescue
        puts "cannot create remote path ${path}"
        exit
      end
    end
  end
  puts
end

####################################################

# find enclosing directory containing bomfile
loop do
  if not Dir.pwd.start_with? local_prefix
    puts "not a mirror directory for #{server}"
    exit
  end
  break if File.exists? bomfile
  Dir.chdir '..'
end
local_path = Dir.pwd
remote_path = local_path.sub local_prefix, remote_prefix

# open sftp session
%x{ ssh-add -K ~/.ssh/id_rsa }
Net::SFTP.start(server, user, port: 2718, keys: keys) do |sftp|
  puts

  # read bomfiles, mark remote bomfile for replacement
  # prepare full upload if remote path doesn't exist
  begin
    sftp.stat! remote_path
    sftp.download!("#{remote_path}/#{bomfile}", bomfile)
    remote_files, remote_dirs = list_bom(bomfile)
    remote_files[bomfile] = nil
  rescue
    remote_files = {}
    remote_dirs = []
    mkdir(sftp, remote_path)
  end
  %x{ mkbom . #{bomfile} }
  local_files, local_dirs = list_bom(bomfile)

  # delete remote files not local (except .mp4 videos)
  (remote_files.keys - local_files.keys).each do |file|
    next if file.end_with? '.mp4'
    update('-', file) do
      sftp.remove! "#{remote_path}/#{file}"
    end
  end

  # delete remove dirs not local
  (remote_dirs - local_dirs).reverse.each do |dir|
    update('-', dir) do
      sftp.rmdir! "#{remote_path}/#{dir}"
    end
  end

  # create remote dirs
  (local_dirs - remote_dirs).each do |dir|
    update('+', dir) do
      sftp.mkdir! "#{remote_path}/#{dir}"
    end
  end

  # upload new or changed local files, and command line arguments
  (local_files.keys | ARGV).each do |file|
    if ARGV.include? file
      tick = '!'
    elsif remote_files.key? file and remote_files[file] == local_files[file]
      next
    else
      tick = remote_files.key?(file) ? tick = '^' : tick = '+'
    end
    update(tick, file) do
      sftp.upload!("#{local_path}/#{file}", "#{remote_path}/#{file}")
    end
  end

end
